﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Data.Objects;
using System.Data;
using System.Diagnostics;
using System.Data.Objects.DataClasses;

namespace Perseus
{
    [DataContract]
    internal class RelationshipEntry
    {
        //
        // serializable properties
        //

        [DataMember] string RelationshipName { get; set; }
        [DataMember] EntityState State { get; set; }

        [DataMember] string Role1 { get; set; }
        [DataMember] EntityKey Key1 { get; set; }
        [DataMember] int AddedEntityIndex1 { get; set; }

        [DataMember] string Role2 { get; set; }
        [DataMember] EntityKey Key2 { get; set; }
        [DataMember] int AddedEntityIndex2 { get; set; }

        //
        // non-serializable properties
        //

        IEntityWithRelationships Entity1 { get; set; }
        IEntityWithRelationships Entity2 { get; set; }

        //
        // constructor
        //

        internal RelationshipEntry(ObjectStateEntry stateEntry, ObjectContext context, 
                                   Dictionary<EntityKey,int> addedEntityKeyToIndex)
        {
            Debug.Assert(stateEntry.IsRelationship);

            this.RelationshipName = stateEntry.EdmType().FullName;
            this.State = stateEntry.State;

            this.Role1 = stateEntry.UsableValues().GetName(0);
            this.Key1 = (EntityKey)stateEntry.UsableValues().GetValue(0);
            if (context.GetEntityState(this.Key1) == EntityState.Added)
            {
                this.AddedEntityIndex1 = addedEntityKeyToIndex[this.Key1];
                this.Key1 = null;
            }

            this.Role2 = stateEntry.UsableValues().GetName(1);
            this.Key2 = (EntityKey)stateEntry.UsableValues().GetValue(1);
            if (context.GetEntityState(this.Key2) == EntityState.Added)
            {
                this.AddedEntityIndex2 = addedEntityKeyToIndex[this.Key2];
                this.Key2 = null;
            }
        }

        //
        // methods
        //

        internal void AddRelationship(ObjectContext context, List<IEntityWithKey> addedEntities)
        {
            Debug.Assert(this.State == EntityState.Added);
            ResolveEntitiesAndKeys(context, addedEntities);

            IRelatedEnd relatedEnd = this.Entity1.RelationshipManager.GetRelatedEnd(this.RelationshipName, this.Role2);
            if (!relatedEnd.Contains(this.Key2))
            {
                relatedEnd.Add(this.Entity2);
            }
        }

        internal void AttachRelationship(ObjectContext context)
        {
            Debug.Assert((this.State == EntityState.Deleted) || (this.State == EntityState.Unchanged));
            // Unchanged and deleted relationships cannot involve added entities, so no need for addedEntities list.
            ResolveEntitiesAndKeys(context, null);

            IRelatedEnd relatedEnd = this.Entity1.RelationshipManager.GetRelatedEnd(this.RelationshipName, this.Role2);
            if (!relatedEnd.Contains(this.Key2))
            {
                relatedEnd.Attach(this.Entity2);
            }
        }

        internal void DeleteRelationship(ObjectContext context)
        {
            Debug.Assert(this.State == EntityState.Deleted);
            // DeletedRelationships cannot involve added entities, so no need for addedEntities list
            ResolveEntitiesAndKeys(context, null);

            IRelatedEnd relatedEnd = this.Entity1.RelationshipManager.GetRelatedEnd(this.RelationshipName, this.Role2);
            if (relatedEnd.Contains(this.Key2))
            {
                relatedEnd.Remove(this.Entity2);
            }
        }
            
        void ResolveEntitiesAndKeys(ObjectContext context, List<IEntityWithKey> addedEntities)
        {
            if (this.Key1 == null)
            {
                this.Entity1 = (IEntityWithRelationships)addedEntities[this.AddedEntityIndex1];
                this.Key1 = ((IEntityWithKey)this.Entity1).EntityKey;
            }
            else
            {
                this.Entity1 = (IEntityWithRelationships)context.GetEntityByKey(this.Key1);
            }

            if (this.Key2 == null)
            {
                this.Entity2 = (IEntityWithRelationships)addedEntities[this.AddedEntityIndex2];
                this.Key2 = ((IEntityWithKey)this.Entity2).EntityKey;
            }
            else
            {
                this.Entity2 = (IEntityWithRelationships)context.GetEntityByKey(this.Key2);
            }
        }   
    }
}
