﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using nwModel;
using System.Diagnostics;

namespace NwindEdmCS
{
    public partial class NwindEdm : Form
    {
        nwEntities ctxNwind = null;
        Stopwatch swTimer = new Stopwatch();
        bool isLoaded = false;
        List<Customer> customerList = null;
        public NwindEdm()
        {
            InitializeComponent();
        }

        private void NwindEdm_Load(object sender, EventArgs e)
        {
            // Load persisted CheckBox state
            chkQueryTimeOnly.Checked = Properties.Settings.Default.qryTimeOnly;
            chkUseOldQuery.Checked = Properties.Settings.Default.useOldQuery;
            chkCacheOrders.Checked = Properties.Settings.Default.cacheOrders;
            
            swTimer.Start();
            ctxNwind = new nwEntities();
            if (chkCacheOrders.Checked)
            {
                // Explicit early loading of Order EntitySets, cache in a List<T>
                var query = ctxNwind.Customers
                            .Include("Orders")
                            .OrderBy(c => c.CustomerID);
                customerList = query.ToList();
                customerBindingSource.DataSource = query.ToList();
            }
            else
            {
                // No explicit EntitySet loading
                customerBindingSource.DataSource = ctxNwind.Customers;
            }

            txtTime.Text = (swTimer.ElapsedMilliseconds / 1000D).ToString("0.000");
            swTimer.Reset();
            if (!isLoaded)
            {
                // Bind the text boxes once (moved from NwindEdm.Designer.cs)
                this.addressTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "Address", true));
                this.cityTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "City", true));
                this.companyNameTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "CompanyName", true));
                this.contactNameTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "ContactName", true));
                this.contactTitleTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "ContactTitle", true));
                this.countryTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "Country", true));
                this.customerIDTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "CustomerID", true));
                this.faxTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "Fax", true));
                this.phoneTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "Phone", true));
                this.postalCodeTextBox.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.customerBindingSource, "PostalCode", true));
            }
            isLoaded = true;
            
            // Load the DataGridView
            customerBindingSource_CurrentChanged(null, null);
        }

        private void customerBindingSource_CurrentChanged(object sender, EventArgs e)
        {
            if (customerIDTextBox.Text.Length == 5) 
            {
                swTimer.Start();
                if (chkUseOldQuery.Checked)
                {
                    // LINQ to Entities query
                    var query = customerList
                                .Where(cust => cust.CustomerID == customerIDTextBox.Text)
                                .Select(c => c.Orders
                                .Select(o => o))
                                .FirstOrDefault()
                                .OrderByDescending(o => o.OrderDate);
                                orderBindingSource.DataSource = query.ToList();
                    
                    if (chkQueryTimeOnly.Checked)
                    {
                        swTimer.Stop();
                    }
                }
                else
                {
                    if (chkCacheOrders.Checked)
                    {
                        // Customers and Orders are should be in the object cache
                        var query = ctxNwind.Orders
                                    .Where(o => o.Customers.CustomerID == customerIDTextBox.Text)
                                    .OrderByDescending(o => o.OrderDate);
                        orderBindingSource.DataSource = query.ToList();
                    }
                    else
                    {
                        // "Correct" version with explicit eager loading 
                        var query = ctxNwind.Orders.Include("Customers")
                                    .Where(o => o.Customers.CustomerID == customerIDTextBox.Text)
                                    .OrderByDescending(o => o.OrderDate);
                        orderBindingSource.DataSource = query.ToList();
                    }
                    if (chkQueryTimeOnly.Checked)
                    {
                        swTimer.Stop();
                    }
                }
                if (isLoaded)
                {
                    txtTime.Text = (swTimer.ElapsedMilliseconds / 1000D).ToString("0.000");
                    swTimer.Reset();
                }
            }
        }

        private void btnSaveChanges_Click(object sender, EventArgs e)
        {
            ctxNwind.SaveChanges(true);
            NwindEdm_Load(null, null);
        }

        private void chkQueryTimeOnly_CheckedChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.qryTimeOnly = chkQueryTimeOnly.Checked;
            Properties.Settings.Default.Save();
        }

        private void chkUseOldQuery_CheckedChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.useOldQuery = chkUseOldQuery.Checked;
            Properties.Settings.Default.Save();
        }

        private void chkCacheOrders_CheckedChanged(object sender, EventArgs e)
        {
            Properties.Settings.Default.cacheOrders = chkCacheOrders.Checked;
            Properties.Settings.Default.Save();
        }
    }
}
